ViperWatch = LibStub("AceAddon-3.0"):NewAddon("ViperWatch", "AceConsole-3.0", "AceEvent-3.0")
ViperWatch.low_mana = 0.1
ViperWatch.high_mana = 1.0
ViperWatch.play_sound = true
ViperWatch.show_low = true
ViperWatch.show_high = true
local L = LibStub("AceLocale-3.0"):GetLocale("ViperWatch", true)

local has_displayed_low_message = false
local has_displayed_high_message = false

local options = {
	name = "ViperWatch",
	handler = ViperWatch,
	type = "group",
	args = {
		low_mana = {
			name = L["LOW_NAME"],
			desc = L["LOW_DESC"],
			type = "range",
			min = 0,
			max = 1.0,
			step = 0.01,
			isPercent = true,
			set = "SetLowMana",
			get = "GetLowMana",
		},
		high_mana = {
			name = L["HIGH_NAME"],
			desc = L["HIGH_DESC"] ,
			type = "range",
			min = 0,
			max = 1.0,
			step = 0.01,
			isPercent = true,
			set = "SetHighMana",
			get = "GetHighMana",
		},
		play_sound = {
			name = L["AUDIO_NAME"],
			desc = L["AUDIO_DESC"],
			type = "toggle",
			get = "ShouldPlaySoundEffect",
			set = "TogglePlaySoundEffect",
		},
		show_low = {
			name = L["SHOW_LOW_NAME"],
			desc = L["SHOW_LOW_DESC"],
			type = "toggle",
			get = "ShouldShowLow",
			set = "ToggleShowLow",
			width = "double",
		},
		show_high = {
			name = L["SHOW_HIGH_NAME"],
			desc = L["SHOW_HIGH_DESC"],
			type = "toggle",
			get = "ShouldShowHigh",
			set = "ToggleShowHigh",
			width = "double",
		},
	},
}

local defaults = {
	profile = {
		high_mana = 1.0,
		low_mana = 0.1,
		play_sound = true,
		show_low = true,
		show_high = true,
	},
}

function ViperWatch:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ViperWatchDB", defaults)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ViperWatch", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ViperWatch", "ViperWatch")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ViperWatch-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))
	self.profileFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ViperWatch-Profiles", "Profiles", "ViperWatch")
	self:RegisterChatCommand("viperwatch", "ChatCommand")
	self:RegisterChatCommand("vw", "ChatCommand")
end

function ViperWatch:OnEnable()
	self:RegisterEvent("UNIT_MANA")
	self:RegisterEvent("UNIT_AURA")
end

function ViperWatch:UNIT_AURA(event_name, unit)
	if unit == "player" then
		mana_pct = (UnitMana(unit) / UnitManaMax(unit))
		if mana_pct >= self.db.profile.high_mana and has_displayed_high_message then
		-- We have high mana - do we have Aspect of the Viper on?
			i = 1
			has_viper = false
			buff  = UnitBuff(unit, i, true)
			while buff do
				if buff == L["VIPER"] then
					-- We have viper on
					has_viper = true
				end
				i = i + 1
				buff = UnitBuff(unit, i, true)
			end
			if not has_viper then
				has_displayed_high_message = false
			end
		end
	end
end

function ViperWatch:UNIT_MANA(event_name, unit)
	if unit == "player" then
		_, class = UnitClass(unit)
		if class == "HUNTER" then
			mana_pct = (UnitMana(unit) / UnitManaMax(unit))
			-- First check to see if we're getting low
			if mana_pct <= self.db.profile.low_mana and self.db.profile.show_low then
				-- We have low mana - do we have Aspect of the Viper on?
				i = 1
				has_viper = false
				buff  = UnitBuff(unit, i, true)
				while buff do
					if buff == L["VIPER"] then
						-- We have viper on
						has_viper = true
					end
					i = i + 1
					buff = UnitBuff(unit, i, true)
				end
				if not has_viper and not has_displayed_low_message and not UnitIsDead(unit) then
					-- We have low mana and no aspect of the viper and need to display - message time!
					UIErrorsFrame:AddMessage(L["LOW"], 1.0, 1.0, 1.0, 5.0)
					if self.db.profile.play_sound then 
						PlaySound("igQuestFailed")
					end
					has_displayed_low_message = true
				end
			end
			-- Next see if we're over or at our threshold
			if mana_pct >= self.db.profile.high_mana and self.db.profile.show_high then
				-- We have high mana - do we have Aspect of the Viper on?
				i = 1
				has_viper = false
				buff  = UnitBuff(unit, i, true)
				while buff do
					if buff == L["VIPER"] then
						-- We have viper on
						has_viper = true
					end
					i = i + 1
					buff = UnitBuff(unit, i, true)
				end
				if has_viper and not has_displayed_high_message and not UnitIsDead(unit) then
					-- We have mana and aspect of the viper - message time!
					UIErrorsFrame:AddMessage(L["HIGH"], 1.0, 1.0, 1.0, 5.0)
					if self.db.profile.play_sound then 
						PlaySound("igQuestFailed")
					end
					has_displayed_high_message = true
				end
			end
			-- Clear it up if our mana is in a safe space.
			if mana_pct > self.db.profile.low_mana and has_displayed_low_message then
				has_displayed_low_message = false
			end
		end
	end
end

function ViperWatch:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory("ViperWatch")
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(ViperWatch, "vw", "ViperWatch", input)
	end
end

function ViperWatch:GetHighMana(info)
	return self.db.profile.high_mana
end

function ViperWatch:SetHighMana(info, new_value)
	self.db.profile.high_mana = new_value
end

function ViperWatch:GetLowMana(info)
	return self.db.profile.low_mana
end

function ViperWatch:SetLowMana(info, new_value)
	self.db.profile.low_mana = new_value
end

function ViperWatch:ShouldPlaySoundEffect(info)
	return self.db.profile.play_sound
end

function ViperWatch:TogglePlaySoundEffect(info, new_value)
	self.db.profile.play_sound = new_value
end

function ViperWatch:ShouldShowLow(info)
	return self.db.profile.show_low
end

function ViperWatch:ToggleShowLow(info, new_value)
	self.db.profile.show_low = new_value
end

function ViperWatch:ShouldShowHigh(info)
	return self.db.profile.show_high
end

function ViperWatch:ToggleShowHigh(info, new_value)
	self.db.profile.show_high = new_value
end